/* SPDX-License-Identifier: (LGPL-2.1 OR BSD-2-Clause) */

/* THIS FILE IS AUTOGENERATED! */
#ifndef __ATOMIC_BOUNDS_SKEL_H__
#define __ATOMIC_BOUNDS_SKEL_H__

#include <errno.h>
#include <stdlib.h>
#include <bpf/libbpf.h>

struct atomic_bounds {
	struct bpf_object_skeleton *skeleton;
	struct bpf_object *obj;
	struct {
		struct bpf_map *data;
	} maps;
	struct {
		struct bpf_program *sub;
	} progs;
	struct {
		struct bpf_link *sub;
	} links;
	struct atomic_bounds__data {
		_Bool skip_tests;
	} *data;
};

static void
atomic_bounds__destroy(struct atomic_bounds *obj)
{
	if (!obj)
		return;
	if (obj->skeleton)
		bpf_object__destroy_skeleton(obj->skeleton);
	free(obj);
}

static inline int
atomic_bounds__create_skeleton(struct atomic_bounds *obj);

static inline struct atomic_bounds *
atomic_bounds__open_opts(const struct bpf_object_open_opts *opts)
{
	struct atomic_bounds *obj;
	int err;

	obj = (struct atomic_bounds *)calloc(1, sizeof(*obj));
	if (!obj) {
		errno = ENOMEM;
		return NULL;
	}

	err = atomic_bounds__create_skeleton(obj);
	err = err ?: bpf_object__open_skeleton(obj->skeleton, opts);
	if (err)
		goto err_out;

	return obj;
err_out:
	atomic_bounds__destroy(obj);
	errno = -err;
	return NULL;
}

static inline struct atomic_bounds *
atomic_bounds__open(void)
{
	return atomic_bounds__open_opts(NULL);
}

static inline int
atomic_bounds__load(struct atomic_bounds *obj)
{
	return bpf_object__load_skeleton(obj->skeleton);
}

static inline struct atomic_bounds *
atomic_bounds__open_and_load(void)
{
	struct atomic_bounds *obj;
	int err;

	obj = atomic_bounds__open();
	if (!obj)
		return NULL;
	err = atomic_bounds__load(obj);
	if (err) {
		atomic_bounds__destroy(obj);
		errno = -err;
		return NULL;
	}
	return obj;
}

static inline int
atomic_bounds__attach(struct atomic_bounds *obj)
{
	return bpf_object__attach_skeleton(obj->skeleton);
}

static inline void
atomic_bounds__detach(struct atomic_bounds *obj)
{
	return bpf_object__detach_skeleton(obj->skeleton);
}

static inline int
atomic_bounds__create_skeleton(struct atomic_bounds *obj)
{
	struct bpf_object_skeleton *s;

	s = (struct bpf_object_skeleton *)calloc(1, sizeof(*s));
	if (!s)
		goto err;
	obj->skeleton = s;

	s->sz = sizeof(*s);
	s->name = "atomic_bounds";
	s->obj = &obj->obj;

	/* maps */
	s->map_cnt = 1;
	s->map_skel_sz = sizeof(*s->maps);
	s->maps = (struct bpf_map_skeleton *)calloc(s->map_cnt, s->map_skel_sz);
	if (!s->maps)
		goto err;

	s->maps[0].name = "atomic_b.data";
	s->maps[0].map = &obj->maps.data;
	s->maps[0].mmaped = (void **)&obj->data;

	/* programs */
	s->prog_cnt = 1;
	s->prog_skel_sz = sizeof(*s->progs);
	s->progs = (struct bpf_prog_skeleton *)calloc(s->prog_cnt, s->prog_skel_sz);
	if (!s->progs)
		goto err;

	s->progs[0].name = "sub";
	s->progs[0].prog = &obj->progs.sub;
	s->progs[0].link = &obj->links.sub;

	s->data_sz = 1216;
	s->data = (void *)"\
\x7f\x45\x4c\x46\x02\x01\x01\0\0\0\0\0\0\0\0\0\x01\0\xf7\0\x01\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\x03\0\0\0\0\0\0\0\0\0\0\x40\0\0\0\0\0\x40\0\x07\0\x01\
\0\0\x2e\x73\x74\x72\x74\x61\x62\0\x2e\x73\x79\x6d\x74\x61\x62\0\x66\x65\x6e\
\x74\x72\x79\x2f\x62\x70\x66\x5f\x66\x65\x6e\x74\x72\x79\x5f\x74\x65\x73\x74\
\x31\0\x2e\x64\x61\x74\x61\0\x61\x74\x6f\x6d\x69\x63\x5f\x62\x6f\x75\x6e\x64\
\x73\x2e\x63\0\x73\x75\x62\0\x73\x6b\x69\x70\x5f\x74\x65\x73\x74\x73\0\x2e\x42\
\x54\x46\0\x2e\x42\x54\x46\x2e\x65\x78\x74\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\x2f\0\0\0\x04\0\xf1\xff\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\x03\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x3f\0\0\0\x12\0\x03\0\0\0\
\0\0\0\0\0\0\x10\0\0\0\0\0\0\0\x43\0\0\0\x11\0\x04\0\0\0\0\0\0\0\0\0\x01\0\0\0\
\0\0\0\0\xb7\0\0\0\0\0\0\0\x95\0\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\x9f\xeb\x01\0\
\x18\0\0\0\0\0\0\0\x84\0\0\0\x84\0\0\0\xe3\0\0\0\0\0\0\0\0\0\0\x02\x02\0\0\0\
\x01\0\0\0\0\0\0\x01\x08\0\0\0\x40\0\0\0\0\0\0\0\x01\0\0\x0d\x04\0\0\0\x14\0\0\
\0\x01\0\0\0\x18\0\0\0\0\0\0\x01\x04\0\0\0\x20\0\0\x01\x1c\0\0\0\x01\0\0\x0c\
\x03\0\0\0\x20\0\0\0\0\0\0\x01\x01\0\0\0\x08\0\0\x04\x26\0\0\0\0\0\0\x0e\x06\0\
\0\0\x01\0\0\0\xc5\0\0\0\x01\0\0\x0f\x01\0\0\0\x07\0\0\0\0\0\0\0\x01\0\0\0\0\
\x75\x6e\x73\x69\x67\x6e\x65\x64\x20\x6c\x6f\x6e\x67\x20\x6c\x6f\x6e\x67\0\x63\
\x74\x78\0\x69\x6e\x74\0\x73\x75\x62\0\x5f\x42\x6f\x6f\x6c\0\x73\x6b\x69\x70\
\x5f\x74\x65\x73\x74\x73\0\x2f\x62\x75\x69\x6c\x64\x64\x69\x72\x2f\x62\x75\x69\
\x6c\x64\x2f\x42\x55\x49\x4c\x44\x2f\x6b\x65\x72\x6e\x65\x6c\x2d\x34\x2e\x31\
\x38\x2e\x30\x2d\x35\x34\x34\x2e\x65\x6c\x38\x2f\x6c\x69\x6e\x75\x78\x2d\x34\
\x2e\x31\x38\x2e\x30\x2d\x35\x34\x34\x2e\x65\x6c\x38\x2e\x61\x61\x72\x63\x68\
\x36\x34\x2f\x74\x6f\x6f\x6c\x73\x2f\x74\x65\x73\x74\x69\x6e\x67\x2f\x73\x65\
\x6c\x66\x74\x65\x73\x74\x73\x2f\x62\x70\x66\x2f\x70\x72\x6f\x67\x73\x2f\x61\
\x74\x6f\x6d\x69\x63\x5f\x62\x6f\x75\x6e\x64\x73\x2e\x63\0\x69\x6e\x74\x20\x42\
\x50\x46\x5f\x50\x52\x4f\x47\x28\x73\x75\x62\x2c\x20\x69\x6e\x74\x20\x78\x29\0\
\x2e\x64\x61\x74\x61\0\x66\x65\x6e\x74\x72\x79\x2f\x62\x70\x66\x5f\x66\x65\x6e\
\x74\x72\x79\x5f\x74\x65\x73\x74\x31\0\0\x9f\xeb\x01\0\x20\0\0\0\0\0\0\0\x14\0\
\0\0\x14\0\0\0\x1c\0\0\0\x30\0\0\0\0\0\0\0\x08\0\0\0\xcb\0\0\0\x01\0\0\0\0\0\0\
\0\x05\0\0\0\x10\0\0\0\xcb\0\0\0\x01\0\0\0\0\0\0\0\x31\0\0\0\xac\0\0\0\x05\x38\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\0\x03\0\0\0\x20\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x40\0\0\0\0\0\0\0\x5c\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x09\0\0\0\x02\0\0\0\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\0\xa0\0\0\0\0\0\0\0\x78\0\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\x08\0\0\0\0\
\0\0\0\x18\0\0\0\0\0\0\0\x11\0\0\0\x01\0\0\0\x06\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\
\x18\x01\0\0\0\0\0\0\x10\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\x29\0\0\0\x01\0\0\0\x03\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x28\x01\0\0\0\
\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x4e\
\0\0\0\x01\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x30\x01\0\0\0\0\0\0\x7f\x01\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\x53\0\0\0\x01\0\0\
\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\xb0\x02\0\0\0\0\0\0\x50\0\0\0\0\0\0\0\0\0\0\
\0\0\0\0\0\x08\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";

	return 0;
err:
	bpf_object__destroy_skeleton(s);
	return -ENOMEM;
}

#endif /* __ATOMIC_BOUNDS_SKEL_H__ */
